/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2009-2014 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package javax.mail.internet;

import org.junit.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

/**
 * Test that non-ASCII file names are encoded by default.
 */
public class NonAsciiFileNames {

    private static String charset;

    @BeforeClass
    public static void before() {
	System.out.println("NonAsciiFileNames");
	charset = System.getProperty("mail.mime.charset");
	System.setProperty("mail.mime.charset", "utf-8");
    }

    /**
     * Test that non-ASCII filenames are encoded by default.
     */
    @Test
    public void testNonAsciiFileName() throws Exception {
	MimeBodyPart mbp = new MimeBodyPart();
	mbp.setText("test\n");
	mbp.setFileName("test\u00a1\u00a2\u00a3");
	MimeBodyPart.updateHeaders(mbp);

	String s = mbp.getHeader("Content-Disposition", null);
	assertTrue("Content-Disposition filename", s.indexOf("filename*") >= 0);
	s = mbp.getHeader("Content-Type", null);
	assertTrue("Content-Type name", s.indexOf("name*") >= 0);
    }

    /**
     * Test that non-ASCII filenames are encoded by default.
     * Make sure an existing Content-Type header is updated.
     */
    @Test
    public void testNonAsciiFileNameWithContentType() throws Exception {
	MimeBodyPart mbp = new MimeBodyPart();
	mbp.setText("test\n");
	mbp.setHeader("Content-Type", "text/x-test");
	mbp.setFileName("test\u00a1\u00a2\u00a3");
	MimeBodyPart.updateHeaders(mbp);

	String s = mbp.getHeader("Content-Disposition", null);
	assertTrue("Content-Disposition filename", s.indexOf("filename*") >= 0);
	s = mbp.getHeader("Content-Type", null);
	assertTrue("Content-Type name", s.indexOf("name*") >= 0);
    }

    @AfterClass
    public static void after() {
	if (charset == null)
	    System.clearProperty("mail.mime.charset");
	else
	    System.setProperty("mail.mime.charset", charset);
    }
}
